#ifndef ___product_generated_h___
#define ___product_generated_h___

#define VBOX_VENDOR "Oracle and/or its affiliates"
#define VBOX_VENDOR_SHORT "Oracle"

#define VBOX_PRODUCT "$(VBOX_VENDOR_SHORT) VirtualBox"
#define VBOX_C_YEAR "2026"

#endif
